# Caleb Hofschneider SLVROV 2025

import atexit
import platform
import subprocess
import sys


def is_raspberry_pi() -> bool:
    """
    Discovers if the current device is a raspberry pi.

    Returns:
        bool: True if raspberry pi, False is not.
    """

    uname = platform.uname()
    return "raspberrypi" in uname.node.lower()

def sys_error(msg: str, exit_code: int=1) -> None:
    print(f"Error: {msg}", file=sys.stderr)
    sys.exit(exit_code)


def get_os():
    os = platform.system()
    if os in ["Darwin", "Linux"]: return os
    else: raise Exception(f"{os} is not supported") 


def at_exit(func):
    """
    Allows a function to be run when the program terminates smoothly.

    Args:
        func (function): The function to be exectued.
    """

    atexit.register(func)


def fits_in_bits(i: int, bits: int, signed: bool | None=None) -> bool:
    """
    Determines if a given int i fits into a given amount of bits.

    Args:
        i (int): The integer in question.
        bits (int): The given amount of bits.
        signed (bool | None): Is the int signed. Default is None, in which case both are tested.

    Returns:
        bool: True if i can be represented by the given number of bits, False if not.
    """

    signed_range = (- (2 ** (bits / 2) - 1), 2 ** (bits / 2))
    unsigned_range = (0, 2 ** bits - 1)

    if signed is None: return signed_range[0] <= i <= signed_range[1] or unsigned_range[0] <= i <= unsigned_range[1]
    elif signed: mn, mx = signed_range
    else: mn, mx = unsigned_range

    return mn <= i <= mx


def safe_run(command: list[str], error_msg: str | None=None, exit_at_exception: bool=False, exit_code: int=1, print_error: bool=True) -> None:

    try:
        subprocess.run(command, check=True)
    except subprocess.CalledProcessError as error:
        msg = error_msg if error_msg is not None else ''
        if print_error: msg += f"\n{error}"

        if exit_at_exception: sys_error(msg, exit_code)
        else: print(msg)


#!/usr/bin/env python3
# Caleb Hofschneider SLVROV 12/2025

import argparse
from ipaddress import ip_address
from ..misc_tools import sys_error
from ..network_tools import has_NetworkManager, has_networkd, NetworkManager_modify_network, networkd_set_ip

RESERVED_START = "192.168.3.2"
RESERVED_END = "192.168.3.13"
CONNECTION_NAME = "Wired Connection 1"
BASE = "192.168.3"


def main() -> None:
    parser = argparse.ArgumentParser(description="Assign a static IPv4 address using nmcli with safety checks.")

    parser.add_argument("address", help="IPV4 address to assign to the interface")
    parser.add_argument("--connection", default=CONNECTION_NAME, help=f"Name of network interface. Default is {CONNECTION_NAME}")
    parser.add_argument("--override", action="store_true", help="Disable ROV address checks")

    args = parser.parse_args()

    try:
        addr = ip_address(args.address)
    except ValueError:
        sys_error(f"Invalid IP address: {args.address}")

    if ip_address(RESERVED_START) <= addr <= ip_address(RESERVED_END): sys_error(f"IP addresses {RESERVED_START} through {RESERVED_END} are reserved")
    if args.address == "192.168.3.0": sys_error(f"192.168.3.0 is special -- don't use it")
    if not args.address.startswith(BASE) and not args.override: sys_error(f"ROV is currently running on {BASE} IPs.")

    if has_NetworkManager: NetworkManager_modify_network(args.address, args.connection)
    elif has_networkd: networkd_set_ip(args.address, args.connection)
    else: sys_error("No supported network backend. Only work for NetworkManager or networkd")


if __name__ == "__main__":
    main()



# SLVROV Jan 2025

import cv2
import pathlib
import time


def cv2_cam(camera_index: int=0) -> cv2.VideoCapture:
    cam = cv2.VideoCapture(camera_index)
    if not cam.isOpened(): raise Exception("Camera not opened")

    return cam


def warm_up_camera(cv2_capture: cv2.VideoCapture, count: int = 5, wait: int=0.1) -> None:
    for _ in range(count):
        ret, frame = cv2_capture.read()
        if not ret: print("Warning: Could not read frame during warmup.")
        time.sleep(wait)


def save_pictures(cv2_capture: cv2.VideoCapture, path: pathlib.PosixPath | str=pathlib.Path("images/img"), count: int=1) -> None:
    """Capture and save a specified number of images from a VideoCapture source.

    Args:
        cv2_capture (cv2.VideoCapture): 
            An active OpenCV video capture object from which frames will be read.
        path (Path | str, optional): 
            Base file path (without index or extension) where images will be saved.
            For example, Path("images/img") will generate files like img1.jpg, img2.jpg, etc.
            The parent directory is created if it does not already exist.
        count (int, optional): 
            Number of images to capture and save. Defaults to 1.

    Raises:
        Exception: If 'path' argument type is invalid
        Exception: If a frame cannot be captured from the video source.

    Code adapted from Tommy Fydrich
    """

    if type(path) == str: path = pathlib.Path(path)

    directory = path.parent
    directory.mkdir(parents=True, exist_ok=True)

    for i in range(count):
        imgpath = path.with_name(f"{path.name}{i + 1}.jpg")
        ret, frame = cv2_capture.read()

        if not ret: raise Exception(f"Error capturing frame {i + 1}")
        cv2.imwrite(str(imgpath), frame)


#!/usr/bin/env python3
# Caleb Hofschneider SLVROV 12/2025

import argparse
from ..gst_tools import gst_install, gst_stream, gst_recieve
from ..misc_tools import sys_error

DEVICE="0"
FRAMERATE="30/1"
WXH = "1280x720"


def main() -> None:

    parser = argparse.ArgumentParser(description="A script to simplify the long and verbose gstreamer commands we use in ROV")

    parser.add_argument("-ip", help="IPV4 address to stream to")
    parser.add_argument("--port", "-p", type=int, help="Port to stream to or recieve from")
    parser.add_argument("--camera-index", "-c", default=DEVICE, type=int, help=f"Camera device index. Default is {DEVICE}")
    parser.add_argument("--dimensions", "-d", default=WXH, help=f"Video dimensions WidthxHeight. Default is {WXH}")
    parser.add_argument("--framerate", "-f", default=FRAMERATE, help=f"Camera framerate. Default is {FRAMERATE}")

    parser.add_argument("--stream", "-s", action="store_true", help="Flag to indicate stream action")
    parser.add_argument("--recieve", "-r", action="store_true", help="Flag to indicate recieve action")
    parser.add_argument("--install", "-i", action="store_true", help="Flag to indicate install action - MUST BE RUN TO STREAM OR RECIEVE. BREW IS REQUIRED ON MACOS")

    args = parser.parse_args()

    if args.stream:
        if args.recieve: sys_error("Cannot preform stream action and recieve action at the same time")
        if not args.ip or not args.port: sys_error("Both an IP address and port number are required for the stream action")

        gst_stream(args.ip, args.port, args.camera_index, args.dimensions, args.framerate)

    elif args.recieve:
        if args.stream: sys_error("Cannot preform recieve action and stream action at the same time")
        if not args.port: sys_error("Port number is required for recieving streams")

        gst_recieve(args.port)

    elif args.install: gst_install()
    else: sys_error("Action needed. Type '--help' or '-h' for usage")
    

if __name__ == "__main__":
    main()


#!/usr/bin/env python3
# Caleb Hofschneider SLVROV 1/2025

import argparse
from ..misc_tools import safe_run

"https://github.com/LegionaryOfLogic/slvrov-tools/"

def main() -> None:
    
    parser = argparse.ArgumentParser(description="A script to download ROS2 Ubuntu images for ROV")

    parser.add_argument("name", type=str, choices=[], help="")



if __name__ == "__main__":
    main()



# SLVROV Dec 2025

from .misc_tools import get_os, safe_run


def gst_install() -> None:
    if get_os() == "Darwin":
        print("Installing gstreamer on MacOS using homebrew...")

        command = ["brew", 
             "install", 
             "gstreamer"]
        
    else:
        print("Installing gstreamer on Linux using sudo apt install...")

        command = ["sudo", 
             "apt", 
             "install", 
             "gstreamer1.0-tools", 
             "gstreamer1.0-plugins-base", 
             "gstreamer1.0-plugins-good", 
             "gstreamer1.0-plugins-bad", 
             "gstreamer1.0-plugins-ugly", 
             "gstreamer1.0-libav", 
             "v4l-utils"]
        
    safe_run(command, "Error running install command")
        

def gst_stream(ip: str, port: int, device: int, wxh: str, framerate: str) -> None:
    width, height = wxh.split('x')

    if get_os() == "Darwin": 
        print(f"Streaming on MacOs to {ip} at port {port}...")

        command = ["gst-launch-1.0", 
             "avfvideosrc", 
             f"device-index={device}", 
             "!", "video/x-raw,", 
             f"width={width},", 
             f"height={height},", 
             f"framerate={framerate}", 
             "!", "jpegenc", 
             "!", 
             "rtpjpegpay", 
             "!", 
             "udpsink", 
             f"host={ip}", 
             f"port={port}", 
             "sync=false"]

    else:
        print(f"Streaming on Linux to {ip} at port {port}...")

        command = ["gst-launch-1.0",          
            "v4l2src", 
            f"device=/dev/video{device}", 
            "!", 
            f"image/jpeg,width={width},height={height},framerate={framerate}", 
            "!", 
            "rtpjpegpay", 
            "!", 
            "udpsink", 
            f"host={ip}", 
            f"port={port}",
            "sync=false"]
        
    safe_run(command, "Error running stream command")


def gst_recieve(port: int):
    if get_os() == "Darwin":
        print(f"MacOS recieving stream on port {port}...")

        command = ['gst-launch-1.0', 
             'udpsrc', 
             f'port={port}', 
             'caps=application/x-rtp,media=video,encoding-name=JPEG,payload=26', 
             '!', 
             'rtpjpegdepay', 
             '!', 
             'jpegdec', 
             '!', 
             'autovideosink', 
             'sync=false']

    else:
        print(f"Linux recieving stream on port {port}...")

        command = ["gst-launch-1.0",
                   "udpsrc",
                   f"port={port}",
                   "caps=application/x-rtp,encoding-name=JPEG,payload=26",
                   "!",
                   "rtpjpegdepay",
                   "!",
                   "jpegdec",
                   "!",
                   "autovideosink",
                   "sync=false"
]
        
    safe_run(command, "Error running recieve command")